@@ -100,7 +100,7 @@ bool pinned() const
 }
 
 
-#if (NTL_CXX_STANDARD >= 2011)
+#if (NTL_CXX_STANDARD >= 2011 && !defined(NTL_DISABLE_MOVE))
 
 ZZ(ZZ&& a) NTL_FAKE_NOEXCEPT
 {
@@ -242,7 +242,6 @@ inline ZZ to_ZZ(unsigned long a) { return ZZ(INIT_VAL, a); }
 inline void conv(ZZ& x, unsigned int a) { _ntl_guintoz((unsigned long)(a), &x.rep); }
 inline ZZ to_ZZ(unsigned int a) { return ZZ(INIT_VAL, a); }
 
-void conv(ZZ& x, const char *s);
 inline ZZ::ZZ(INIT_VAL_TYPE, const char *s)  { conv(*this, s); }
 inline ZZ to_ZZ(const char *s) { return ZZ(INIT_VAL, s); }
 
@@ -1074,7 +1073,7 @@ void DeriveKey(unsigned char *key, long klen,
 #define NTL_PRG_KEYLEN (32)
 
 
-class RandomStream_impl;
+struct RandomStream_impl;
 
 RandomStream_impl *
 RandomStream_impl_build(const unsigned char *key);
@@ -1095,6 +1094,8 @@ long
 RandomStream_impl_get_bytes(RandomStream_impl& impl, unsigned char *res, 
    long n, long pos);
 
+void RandomStream_impl_set_nonce(RandomStream_impl& impl, unsigned long nonce);
+
 void
 RandomStream_impl_delete(RandomStream_impl*);
 
@@ -1158,8 +1159,24 @@ public:
       }
    }
 
+   // FIXME: document this? Not sure if I want to 
+   // commit to this interface, as it is somewhat ChaCha-specific
+   void set_nonce(unsigned long nonce) 
+   {
+      RandomStream_impl_set_nonce(*impl, nonce);
+      pos = buf_len;
+   }
+
 };
 
+// this is the number of bits we can pass through the set_nonce
+// interface
+#if (NTL_BITS_PER_LONG > 64)
+#define NTL_BITS_PER_NONCE (64)
+#else
+#define NTL_BITS_PER_NONCE NTL_BITS_PER_LONG
+#endif
+
 
 
 // ============ old stuff: for testing  ============
@@ -1231,7 +1248,6 @@ public:
 
 
 
-
 void RandomBnd(ZZ& x, const ZZ& n);
 // x = "random number" in the range 0..n-1, or 0  if n <= 0
 
@@ -1273,7 +1289,6 @@ unsigned long RandomBits_ulong(long l);
 
 // helper class to make generating small random numbers faster
 // FIXME: add documentation?
-
 struct RandomBndGenerator {
 
    long p;
@@ -1333,6 +1348,8 @@ inline void VectorRandomBnd(long k, long* x, long n)
 }
 
 
+void VectorRandomWord(long k, unsigned long* x);
+
 
 /**********************************************************
 
