@@ -27,13 +27,7 @@
 #endif
 
 
-#if (defined(NTL_THREADS) && defined(__GNUC__) && !defined(NTL_DISABLE_TLS_HACK))
-#define NTL_TLS_HACK
-#endif
-
-
-
-#ifdef NTL_TLS_HACK
+#if (defined(NTL_THREADS) && defined(NTL_TLS_HACK)) 
 #include <pthread.h>
 #endif
 
@@ -254,30 +248,58 @@ inline void swap(long& a, long& b)  {  long t;  t = a; a = b; b = t; }
 inline void swap(int& a, int& b)  {  int t;  t = a; a = b; b = t; }
 
 
+// some convenience casting routines:
+
+inline unsigned long cast_unsigned(long a) { return (unsigned long) a; }
+inline unsigned int cast_unsigned(int a) { return (unsigned int) a; }
+
+
+// these routines respect the NTL_CLEAN_INT flag: if set,
+// they use code that is guaranteed to work, under the
+// assumption that signed integers are two's complement.
+// A good compiler should optimize it all away and generate
+// the same code in either case (tested on gcc, clang, icc, msvc++).
+// This is really an academic exercise...
+
+#ifdef NTL_CLEAN_INT
+
+inline long cast_signed(unsigned long a) 
+{ return NTL_ULONG_TO_LONG(a); }
+
+inline int cast_signed(unsigned int a) 
+{ return NTL_UINT_TO_INT(a); }
+
+#else
+
+inline long cast_signed(unsigned long a) { return long(a); }
+inline int cast_signed(unsigned int a) { return int(a); }
+
+#endif
+
 
 inline void conv(int& x, int a) { x = a; }
 inline void conv(int& x, long a) 
-   { unsigned y = (unsigned) a;  x = NTL_UINT_TO_INT(y); }
+   { unsigned y = (unsigned) a;  x = cast_signed(y); }
 inline void conv(int& x, float a) { x = int(NTL_SNS floor(double(a))); }
 inline void conv(int& x, double a) { x = int(NTL_SNS floor(a)); }
 
 inline void conv(int& x, unsigned a) 
-   { x = NTL_UINT_TO_INT(a); }
+   { x = cast_signed(a); }
 
 inline void conv(int& x, unsigned long a)
-   { unsigned y = (unsigned) a;  x = NTL_UINT_TO_INT(y); }
+   { unsigned y = (unsigned) a;  x = cast_signed(y); }
 
 inline int to_int(int a) { return a; }
 inline int to_int(long a) 
-   { unsigned y = (unsigned) a;  return NTL_UINT_TO_INT(y); }
+   { unsigned y = (unsigned) a;  return cast_signed(y); }
 inline int to_int(float a) { return int(NTL_SNS floor(double(a))); }
 inline int to_int(double a) { return int(NTL_SNS floor(a)); }
 
 inline int to_int(unsigned a) 
-   { return NTL_UINT_TO_INT(a); }
+   { return cast_signed(a); }
 
 inline int to_int(unsigned long a) 
-   { unsigned y = (unsigned) a;  return NTL_UINT_TO_INT(y); }
+   { unsigned y = (unsigned) a;  return cast_signed(y); }
 
 
 inline void conv(long& x, int a) { x = a; }
@@ -286,10 +308,10 @@ inline void conv(long& x, float a) { x = long(NTL_SNS floor(double(a))); }
 inline void conv(long& x, double a) { x = long(NTL_SNS floor(a)); }
 
 inline void conv(long& x, unsigned a)
-   { unsigned long y = a;  x = NTL_ULONG_TO_LONG(y); }
+   { unsigned long y = a;  x = cast_signed(y); }
 
 inline void conv(long& x, unsigned long a)
-   { x = NTL_ULONG_TO_LONG(a); }
+   { x = cast_signed(a); }
 
 inline long to_long(int a) { return a; }
 inline long to_long(long a) { return a; }
@@ -297,10 +319,10 @@ inline long to_long(float a) { return long(NTL_SNS floor(double(a))); }
 inline long to_long(double a) { return long(NTL_SNS floor(a)); }
 
 inline long to_long(unsigned a)
-   { unsigned long y = a;  return NTL_ULONG_TO_LONG(y); }
+   { unsigned long y = a;  return cast_signed(y); }
 
 inline long to_long(unsigned long a)
-   { return NTL_ULONG_TO_LONG(a); }
+   { return cast_signed(a); }
 
 inline void conv(float& x, int a) { x = float(a); }
 inline void conv(float& x, long a) { x = float(a); }
@@ -350,39 +372,6 @@ inline void conv(unsigned long& x, float a) { x = ((unsigned int) to_long(a)); }
 inline void conv(unsigned long& x, double a) { x = ((unsigned int) to_long(a)); }
 
 
-// some convenience casting routines:
-
-inline long cast_signed(unsigned long a) { return long(a); }
-inline int cast_signed(unsigned int a) { return int(a); }
-// DIRT: IMPL-DEF: the behavior here is implementation defined,
-// but on a 2s compliment machine, it should always work
-
-inline unsigned long cast_unsigned(long a) { return (unsigned long) a; }
-inline unsigned int cast_unsigned(int a) { return (unsigned int) a; }
-
-
-// these versions respect the NTL_CLEAN_INT flag: if set,
-// they use code that is guaranteed to work, under the
-// assumption that signed intgers are two's complement.
-// A good compiler should optimize it all away and generate
-// the same code in either case (tested on gcc, clang, icc).
-// This is really an academic exercise...
-
-#ifdef NTL_CLEAN_INT
-
-inline long clean_cast_signed(unsigned long a) 
-{ return NTL_ULONG_TO_LONG(a); }
-
-inline int clean_cast_signed(unsigned int a) 
-{ return NTL_UINT_TO_INT(a); }
-
-#else
-
-inline long clean_cast_signed(unsigned long a) { return long(a); }
-inline int clean_cast_signed(unsigned int a) { return int(a); }
-
-#endif
-
 
 
 
@@ -410,6 +399,8 @@ char IntValToChar(long a);
 inline double GetTime() { return _ntl_GetTime(); }
 inline unsigned long GetPID() { return _ntl_GetPID(); }
 
+inline double GetWallTime() { return _ntl_GetWallTime(); }
+
 inline long IsFinite(double *p) { return _ntl_IsFinite(p); }
 
 
@@ -640,7 +631,9 @@ public:
 
 
 
-#ifdef NTL_TLS_HACK
+#if (defined(NTL_THREADS) && defined(NTL_TLS_HACK)) 
+
+// NOTE: All of this TLS code is replicated in CheckThreads.cpp.
 
 
 namespace details_pthread {
@@ -985,7 +978,7 @@ ll_mul_hi(unsigned long a, unsigned long b)
 #define NTL_DECLARE_RELOCATABLE_WHEN(x) \
 constexpr bool DeclareRelocatableType x
 
-#if (defined(NTL_HAVE_COPY_TRAITS1))
+#if (defined(NTL_HAVE_COPY_TRAITS1) || defined(_MSC_VER))
 
 
 // This strategy is used on compilers that fully support C++11 type traits.
@@ -1005,6 +998,12 @@ constexpr bool Relocate_aux_has_trivial_copy(T*)
            std::is_copy_constructible<T>::value;
 }
 
+template<class T>
+constexpr bool Relocate_aux_has_any_copy(T*)
+{
+   return std::is_copy_constructible<T>::value;
+}
+
 #elif (defined(NTL_HAVE_COPY_TRAITS2))
 
 // This strategy is needed on GCC before v5.0, as the required type
@@ -1048,6 +1047,12 @@ constexpr bool Relocate_aux_has_trivial_copy(T*)
            Relocate_aux_has_copy<T>::value;
 }
 
+template<class T>
+constexpr bool Relocate_aux_has_any_copy(T*)
+{
+   return Relocate_aux_has_copy<T>::value;
+}
+
 
 #else
 
